<?php

/**
 * PHPOF2/Optimisation/mysqli.php - mysqli optimisations for PHPOF2
 *
 * (c)2001-2008 Tim Jackson (tim@timj.co.uk)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU Lesser General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package PHPOF2
 * @subpackage Driver_mysqli
 * 
 */

require_once 'PHPOF2/Optimisation/Interface.php';

class PHPOF2_Optimisation_mysqli implements PHPOF2_Optimisation_Interface
{
	public static function tableDiscoverSchema(PHPOF2_DBTable &$table, &$fields_normal, &$fields_lob, &$pri_keys, &$fgn_keys, &$key_auto_increment)
	{
		$res = $table->db->query("DESCRIBE $table->name");
		if (PEAR::isError($res)) {
    		throw new Exception($res->getMessage());
		}
		
		$autoinc = array();
		while ($field = $res->fetchRow(MDB2_FETCHMODE_ASSOC)) {
			$field = array_change_key_case($field, CASE_LOWER);
			$fieldname = $field['field'];
			if ($field['key'] != 'PRI') {
				if (!stristr($field['type'], 'blob')) {
					$fields_normal[] = $fieldname;
				} else {
					$fields_lob[] = $fieldname;
				}
			} else {
				$pri_keys[] = $field['field'];
				if (stristr($field['extra'], 'auto_increment')) {
					$autoinc[$field['field']] = true; 
				}
			}
		}
	
		// If there is a single primary key and it's auto-incrementing, flag this
		if (count($pri_keys) == 1 && isset($autoinc[$pri_keys[0]])) {
			$key_auto_increment = true;
		}
		
		// Get foreign keys
		$res = $table->db->query('SHOW CREATE TABLE ' . $table->name);
		if (PEAR::isError($res)) {
    		throw new Exception($res->getMessage());
		}
		$field = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
		$field = array_change_key_case($field, CASE_LOWER);
		$create_table = $field['create table'];
		
		//CONSTRAINT `const_name` FOREIGN KEY (`colname1`) REFERENCES `foreign_table` (`foreign_col1`)
		if (preg_match_all('/FOREIGN KEY \(`(.+)`\) REFERENCES `(.+)` \(`(.+)`\)/', $create_table, $matches, PREG_SET_ORDER)) {
			foreach ($matches as $match) {
				$fields = explode(',', $match[1]);
				$referenced_table = $match[2];
				$referenced_fields = explode(',', $match[3]);
				
				foreach ($fields as $i => $field) {
					$field = trim($field);
					$fgn_keys[$field][] = $referenced_table . '.' . trim($referenced_fields[$i]);
				}
			}
		}
		
		// Now create _list properties
		$table->_createSchemaCachedLists();
		
		return;
	}
}
